{% comment %}

Example template

{% endcomment %}

{% load render_table from django_tables2 %}

<h1>{{ self.title }}</h1>

{% for element in elements.values %}
    <h3>{{ element.title }}</h3>

    {% render_table element.table %}
{% empty %}
    No tables defined for this report. Please add some tables in the
    admin interface.
{% endfor %}
