# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-26 08:40
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('flexible_reports', '0009_auto_20171025_0558'),
    ]

    operations = [
        migrations.AddField(
            model_name='reportelement',
            name='base_model',
            field=models.ForeignKey(blank=True, help_text="Fill only if 'except catchall' is selected in 'Data from' field", null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType', verbose_name='Base model'),
        ),
        migrations.AlterField(
            model_name='reportelement',
            name='data_from',
            field=models.IntegerField(choices=[(0, 'datasource'), (2, 'except catchall')], default=0, verbose_name='Data from'),
        ),
        migrations.AlterField(
            model_name='reportelement',
            name='datasource',
            field=models.ForeignKey(blank=True, help_text="Fill only if 'datasource' has been chosen in 'Data from' field", null=True, on_delete=django.db.models.deletion.CASCADE, to='flexible_reports.Datasource', verbose_name='Datasource'),
        ),
    ]
