# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-10-25 10:53
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flexible_reports', '0007_sort_desc'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='columnorder',
            options={'ordering': ('position',), 'verbose_name': 'Column order information', 'verbose_name_plural': 'Column order informations'},
        ),
        migrations.AddField(
            model_name='reportelement',
            name='data_from',
            field=models.IntegerField(choices=[(0, 'datasource'), (1, 'catchall'), (2, 'except catchall')], default=0, verbose_name='Data from'),
        ),
        migrations.AlterField(
            model_name='reportelement',
            name='datasource',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='flexible_reports.Datasource', verbose_name='Datasource'),
        ),
        migrations.AlterField(
            model_name='reportelement',
            name='parent',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='flexible_reports.Report', verbose_name='Parent'),
        ),
        migrations.AlterField(
            model_name='reportelement',
            name='slug',
            field=models.SlugField(max_length=200, verbose_name='Slug'),
        ),
        migrations.AlterField(
            model_name='reportelement',
            name='table',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='flexible_reports.Table', verbose_name='Table'),
        ),
        migrations.AlterUniqueTogether(
            name='reportelement',
            unique_together=set([('parent', 'slug')]),
        ),
    ]
